from bbcloud_python_sdk.Api.Models.Http import Http


class Client():
    def __init__(self):
        self.api_prefix = 'api'

        self._auth_team = None
        self._auth_application = None
        self._auth_namespace = None


    def config(self, host, token):
        self.token = token
        self.host = host
        self.base_url = '%s/%s' % (self.host, self.api_prefix)
        self.protocol = Http(client=self)

    def auth_team(self, auth_team):
        self._auth_team = auth_team

    def auth_application(self, auth_application):
        self._auth_application = auth_application

    def auth_namespace(self, auth_namespace):
        self._auth_namespace = auth_namespace



BBCloud = Client()



import logging
import os
import random
import re
import string

from obs import ObsClient

import bbcloud_python_sdk


class ObsFileCache():
    def __init__(self, access_key_id, secret_access_key, endpoint, bucket_name, cache_path_root):
        self.bucket_name = bucket_name
        self.cache_path_root = cache_path_root
        self.obs = ObsClient(
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            server=endpoint
        )

    def random_str(self, num):
        salt = ''.join(random.sample(string.ascii_letters + string.digits, num))

        return salt

    def set_namespace(self, namespace):
        self.namespace = namespace
        logging.info('setting namespace %s' % namespace)
        return self

    def set(self, key, file_path, del_local=True):
        """
        缓存文件到OBS
        @param key: 缓存键
        @param file_path: 文件路径 url|本地目录|本地文件
        @param del_local: 缓存后是否删除本地文件
        """
        local_path = '/tmp/%s.%s' % (self.random_str(30), self.random_str(30))
        object_key = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        if file_path:
            if isinstance(file_path, str) and re.match("http", file_path) and file_path is not None:
                bbcloud_python_sdk.download_file(url=file_path, save_path=local_path)
            elif isinstance(file_path, dict) or isinstance(file_path, list):
                bbcloud_python_sdk.create_json_file(dst_path=local_path, dist=file_path)
            elif os.path.isdir(file_path):
                bbcloud_python_sdk.zip_dirs(zip_filename=local_path, file_dir=file_path)
            elif os.path.isfile(file_path):
                local_path = file_path

            resp = self.obs.putFile(
                bucketName=self.bucket_name,
                objectKey=object_key,
                file_path=local_path
            )
            if resp.status < 300:
                print('requestId:', resp.requestId)
                print('etag:', resp.body.etag)
                print('versionId:', resp.body.versionId)
                print('storageClass:', resp.body.storageClass)
                if del_local:
                    if os.path.exists(local_path):
                        os.remove(local_path)
                return True
            else:
                print('errorCode:', resp.errorCode)
                print('errorMessage:', resp.errorMessage)
                return False

    def get(self, key, local_file):
        """
        从OBS获取缓存文件
        @param key: 缓存键
        @param local_file: 文本保存路径
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        try:
            resp = self.obs.getObject(
                bucketName=self.bucket_name,
                objectKey=cache_path,
                downloadPath=local_file
            )
            if resp.status < 300:
                print('requestId:', resp.requestId)
                print('url:', resp.body.url)
                return True
            else:
                print('errorCode:', resp.errorCode)
                print('errorMessage:', resp.errorMessage)
                return False
        except:
            import traceback
            print(traceback.format_exc())

    def delete(self, key):
        """
        删除OSS缓存文件
        @param key:
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)
        resp = self.obs.deleteObject(
            bucketName=self.bucket_name,
            objectKey=cache_path
        )
        if resp.status < 300:
            print('requestId:', resp.requestId)
            print('deleteMarker:', resp.body.deleteMarker)
            print('versionId:', resp.body.versionId)
            return True
        else:
            print('errorCode:', resp.errorCode)
            print('errorMessage:', resp.errorMessage)
            return False

    def exist(self, key):
        """
        判断缓存文件是否存在
        :param key:
        :return: True|False
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        resp = self.obs.getObjectMetadata(
            bucketName=self.bucket_name,
            objectKey=cache_path
        )
        if resp.status < 300:
            return True
        else:
            return False

    def list_cache_objects(self):
        """
        获取OBS命名空间下的缓存文件
        """
        prefix_path = "%s/%s" % (self.cache_path_root, self.namespace)
        list_dir = []
        marker = ''
        begin = False
        while (marker != '') or (not begin):
            list_objects = self.obs.listObjects(
                bucketName=self.bucket_name,
                prefix=prefix_path,
                max_keys=100,
                marker=marker
            )
            marker = list_objects.next_marker
            begin = True
            for i in list_objects.contents:
                list_dir.append(i)
        return list_dir

    def list_dir(self, ):
        prefix_path = "%s/%s" % (self.cache_path_root, self.namespace)
        all_list_dir = []
        one_list_dir = []
        marker = ''
        begin = False
        while (marker != '') or (not begin):
            list_objects = self.obs.listObjects(
                bucketName=self.bucket_name,
                prefix=prefix_path,
                max_keys=100,
                marker=marker
            )
            marker = list_objects.next_marker
            begin = True
            for i in list_objects.contents:
                all_list_dir.append(i.key.rsplit("/", 1)[0])
        for data in all_list_dir:
            tmp_dir = data.rsplit("/", len(data.split('/')) - len(prefix_path.split('/')) - deep_num)[0]
            if tmp_dir not in one_list_dir:
                one_list_dir.append(tmp_dir)
        if self.cache_path_root in one_list_dir:
            one_list_dir.remove(self.cache_path_root)
        if prefix_path in one_list_dir:
            one_list_dir.remove(prefix_path)
        return one_list_dir



import logging
import os
import random
import re
import string
import warnings

import bbcloud_python_sdk
import requests
from synology_api import filestation


class SynologyFileCache():

    def __init__(self, ip_address, port, username, password, cache_path_root, namespace=None):
        warnings.simplefilter('ignore', ResourceWarning)
        self.cache_path_root = cache_path_root
        self.namespace = namespace
        self.ip_address = ip_address
        self.port = port
        self.username = username
        self.password = password

    def getFl(self):
        return filestation.FileStation(ip_address=self.ip_address, port=self.port, username=self.username,
                                       password=self.password)

    def random_str(self, num):
        salt = ''.join(random.sample(string.ascii_letters + string.digits, num))

        return salt

    def set_namespace(self, namespace):
        self.namespace = namespace
        return self

    def _exist(self, file_path):
        try:
            res = self.getFl().get_file_info(file_path)
            if res['data']['files'][0]['code'] == 408:
                return False
            elif res['data']['files'][0]['name']:
                return True
        except KeyError:
            return True

    def create_folder(self, folder_path):
        basename = os.path.basename(folder_path)
        dirname = os.path.dirname(folder_path)
        self.getFl().create_folder(folder_path=dirname, name=basename)

    def set(self, key, file_path, del_local=True):
        """
        缓存文件到群晖
        @param key: 缓存键
        @param file_path: 文件路径 url|本地目录|本地文件
        @param del_local: 缓存后是否删除本地文件
        """
        local_path = '/tmp/%s.%s/%s' % (self.random_str(30), self.random_str(30), key)
        cache_path = "%s/%s" % (self.cache_path_root, self.namespace)

        if not self._exist(file_path=cache_path):
            self.create_folder(folder_path=cache_path)

        if not os.path.exists(os.path.dirname(local_path)):
            bbcloud_python_sdk.make_dir(os.path.dirname(local_path))

        if file_path:
            if isinstance(file_path, str) and re.match("http", file_path) and file_path is not None:
                bbcloud_python_sdk.download_file(file_path, local_path)
            elif isinstance(file_path, dict) or isinstance(file_path, list):
                bbcloud_python_sdk.create_json_file(local_path, file_path)
            elif os.path.isdir(file_path):
                bbcloud_python_sdk.zip_dirs(local_path, file_path)
            elif os.path.isfile(file_path):
                new_file_path = os.path.join(os.path.dirname(file_path), key)
                local_path = new_file_path
            res = self.getFl().upload_file(dest_path=cache_path, file_path=local_path)
            logging.info(res)
            if del_local:
                if os.path.exists(local_path):
                    os.remove(local_path)

    def get(self, key, local_file):
        """
        从群晖获取缓存文件
        @param key: 缓存键
        @param local_file: 文本保存路径
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)
        local_file_dirname = os.path.dirname(local_file)

        try:
            self.getFl().get_file(path=cache_path, mode='download', dest_path=local_file_dirname)
            local_file_hash_name = os.path.join(local_file_dirname, key)
            if os.path.exists(local_file_hash_name):
                os.rename(local_file_hash_name, local_file)
                return True
            return False
        except requests.exceptions.HTTPError:
            return False

    def delete(self, key):
        """
        删除群晖缓存文件
        @param key:
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        try:
            return self.getFl().delete_blocking_function(path=cache_path)
        except:
            return False

    def exist(self, key):
        """
        判断缓存文件是否存在
        :param key:
        :return: True|False
        """
        file_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)
        return self._exist(file_path=file_path)



class ParameterErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class MethodNotAllowedErrorException(Exception):
    def __init__(self, method, url):
        super().__init__('%s method not allowed for %s' % (method, url))


class ServiceErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class NetworkErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class NoResourcesException(Exception):
    def __init__(self, message):
        super().__init__(message)

class ClientErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)



from bbcloud_python_sdk.Api.Models.Model import Model


class Cocos2dCompileOrdersItem(Model):
    def __init__(self, *args, **kwargs):
        super(Cocos2dCompileOrdersItem, self).__init__(*args, **kwargs)
        self._modal_path = 'packer/cocos2d_compile_orders_items'



class Query(object):

    def __init__(self):
        self.boot()

    def boot(self):
        self._query = ''
        self._order_by = None
        self._page = 1
        self._page_size = 20

    where_operator_map = {
        '=': 'eq',
        '!=': 'ne',
        '>': 'gt',
        '>=': 'ge',
        '<': 'le',
        '<=': 'lt',
        'like': 'like',
    }

    def has(self, key):
        self._query = '%s%s__has,' % (self._query, key)
        return self

    def whereIn(self, key, value):
        self._query = '%s%s__in=%s,' % (self._query, key, value.split('|'))
        return self

    def whereNotIn(self, key, value):
        self._query = '%s%s__not_in=%s,' % (self._query, key, value.split('|'))
        return self

    def where(self, *args):
        if len(args) == 2:
            self._query = '%s%s=%s,' % (self._query, args[0], args[1])

        elif len(args) == 3:
            self._query = '%s%s__%s=%s,' % (self._query, args[0], self.where_operator_map[args[1]], args[2])

        return self

    def page(self, page=1):
        self._page = page
        return self

    def page_size(self, page_size=20):
        self._page_size = page_size
        return self

    def order_by(self, field, order_by='desc'):
        self._order_by = '%s%s' % ('-' if order_by == 'desc' else '+', field)
        return self
